InstallGlobalFunction(ExampleDerivedLength10Group,
function(n)
    return PcGroupCode(SE_Codes[n], 2^11 * 3^13);
end);

#############################################################################

InstallGlobalFunction(lambda,
function(g)
    local low;

    if IsAbelian(g) then
        return fail;
    fi;

    low := LowerCentralSeries(g);
    return low[Length(low)-1];
end);

InstallGlobalFunction(derived,
function(g)
    local series;

    if IsAbelian(g) then
        return fail;
    fi;

    series := DerivedSeries(g);
    return series[Length(series)-1];
end);

#############################################################################

InstallGlobalFunction(extractOperation,
function(pc, k)
    local mats, i, j, nrg, nre, dim, ser, pcgs;

    ser := DerivedSeries(pc);
    pcgs := Pcgs(pc);

    nrg := Length(GeneratorsOfGroup(pc))-Length(GeneratorsOfGroup(ser[k]));
    nre := Length(GeneratorsOfGroup(pc))-Length(GeneratorsOfGroup(ser[k+1]));

    dim := nre - nrg;

    mats := EmptyPlist(nrg);

    for i in [1..nrg] do
        mats[i] := EmptyPlist(dim);
        for j in [nrg+1..nre] do
            mats[i][j-nrg] := ExponentsOfConjugate(pcgs, j, i){[nrg+1..nre]};
        od;
    od;
    
    return mats;
end);


InstallGlobalFunction(moduleOperations,
function(pc, d, p)
    local gl, mingens, ord, orders, gens, mingroup, cc, reps, repspos,
          elms, sets, ops, trans, filt, poslist, pos, factors;

    trans := function(op)
        local exl, i, k, mats;

        mats := EmptyPlist(Length(factors));
        for k in [1..Length(factors)] do
            mats[k] := One(gl);
            exl := factors[k];
            for i in [2,4..Length(exl)] do
                mats[k] := mats[k]*op[exl[i-1]]^exl[i];
            od;
        od;

        return mats;            
    end;

    gl := GL(d, GF(p));
    cc := ConjugacyClasses(gl);
    reps := List(cc, Representative);

    gens := GeneratorsOfGroup(pc);
    mingens := MinimalGeneratingSet(pc);
    orders := List(mingens, Order);
    sets := EmptyPlist(Length(orders));
    for ord in Unique(orders) do
        repspos := PositionsProperty(reps, rep-> IsInt(ord/Order(rep)) );
        filt := Concatenation( List( cc{repspos}, AsList) );;
        poslist := Positions(orders, ord);
        for pos in poslist do
            sets[pos] := filt;
        od;
    od;

    ops := Filtered(Cartesian(sets),
                    tup-> GroupHomomorphismByImages(pc, gl, mingens, tup) 
                          <> fail
                   );

    mingroup := GroupByGenerators(mingens);
    factors := List(gens, x-> ExtRepOfObj(Factorization(mingroup, x)));

    Apply(ops, trans);

    return ops;
end);

#############################################################################

InstallGlobalFunction(trivialDSE,
function(g)
    local series, len, comm;

    series := DerivedSeries(g);
    len := Length(series);

    if len < 3 then
        return true;
    else
        comm := CommutatorSubgroup(series[len-2], series[len-1]);
        # if it is trivial or equal to the last non-trivial term
        return comm = series[len-1] or comm = series[len];
    fi;
end);
